function Q = FindQ(d, j, normalization)
%
% MathLab set of functions for B-spline wavelets
% From: E.J. Stollnitz, T.D. DeRose, D.H. Salesin. 
%       Computer Graphics. Theory and Applications.
%       Morgan Kaufmann Publishers, Inc., San Francisco, California, 1996.
%
% Function: Q = FindQ(d, j, normalization)
% Purpose:  returns the Q matrix for B-spline scaling functions
%           of degree d, level j. If normalization is 'min' (default)
%           then the smallest entry in each colomn is 1,
%           if normalization is 'max'
%           then the largest entry in each colomn is 1,
%           if normalization is 'L2'
%           then the L2 norm of each wavelet is 1,
% Comments:
%
if nargin < 3
   normalization = 'min';
elseif ~strcmp(normalization, 'min') & ~strcmp(normalization, 'max') ...
      & ~strcmp(normalization, 'L2')
   error('FindQ: normalization must be ''min'' or ''max'' or ''L2''.');
end;
P = FindP(d, j);
I = Inner(d, j);
M = P' * I;
[m1, m2] = size(M);
n = m2 - rank(M);
Q = zeros(m2, n);
found = 0;
start_col = 0;
while (found < n / 2) & (start_col < m2)
   start_col = start_col + 1 + (found > d);
   width = 0;
   rank_def = 0;
   while ~rank_def & (width < m2 - start_col + 1)
      width = width + 1;
      submatrix = M(:, start_col : start_col + width - 1);
      rank_def = width - rank(submatrix);
   end;
   if rank_def
      % find null-space of matrix (should be just 1 colomn)
      q_col = null(submatrix);
      if strcmp(normalization, 'min')
         q_col = q_col / min(abs(q_col + 1e38 * (abs(q_col) < 1e-10)));
      elseif strcmp(normalization, 'max')
         q_col = q_col / max(abs(q_col));
      end;
      %change sign to give consistent orientation
      q_col = q_col * (-1) ^ (start_col + floor((d + 1) / 2) + (q_col(1, 1) > 0));
      %put colomn into left half of Q
      found = found + 1;
      Q(start_col : start_col + width - 1, found) = q_col;
      %use symmetry to put colomn into right part of Q
      %in inverse order and negated if degree is even
      Q( :, n - found + 1) = flipud(Q(:, found)) * (-1) ^ (d + 1);
   end;
end;
if (strcmp(normalization, 'L2'))
   ip = Q' * I * Q;
   Q = Q * diag(1 ./ sqrt(diag(ip)));
end;
return;
% ... FindQ()
